@@ -88,11 +88,6 @@ void mbtowc_reset(void)
     IGNORE_CALL_RESULT(mbtowc(NULL, NULL, 0));
 }
 
-void wctomb_reset(void)
-{
-    IGNORE_CALL_RESULT(wctomb(NULL, 0));
-}
-
 /* This function is equivalent to isalpha() for multibyte characters. */
 bool is_alpha_mbchar(const char *c)
 {
@@ -172,8 +167,6 @@ bool is_cntrl_char(int c)
  * their high bits set. */
 bool is_cntrl_mbchar(const char *c)
 {
-    assert(c != NULL);
-
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	return ((c[0] & 0xE0) == 0 || c[0] == 127 ||
@@ -208,8 +201,6 @@ bool is_punct_mbchar(const char *c)
  * punctuation when allow_punct is TRUE), and FALSE otherwise. */
 bool is_word_mbchar(const char *c, bool allow_punct)
 {
-    assert(c != NULL);
-
     if (*c == '\0')
 	return FALSE;
 
@@ -217,15 +208,11 @@ bool is_word_mbchar(const char *c, bool allow_punct)
 	return TRUE;
 
     if (word_chars != NULL && *word_chars != '\0') {
-	bool wordforming;
-	char *symbol = charalloc(MB_CUR_MAX + 1);
+	char symbol[mb_cur_max() + 1];
 	int symlen = parse_mbchar(c, symbol, NULL);
 
 	symbol[symlen] = '\0';
-	wordforming = (strstr(word_chars, symbol) != NULL);
-	free(symbol);
-
-	return wordforming;
+	return (strstr(word_chars, symbol) != NULL);
     }
 
     return (allow_punct && is_punct_mbchar(c));
@@ -234,12 +221,7 @@ bool is_word_mbchar(const char *c, bool allow_punct)
 /* Return the visible representation of control character c. */
 char control_rep(const signed char c)
 {
-    assert(is_cntrl_char(c));
-
-    /* An embedded newline is an encoded null. */
-    if (c == '\n')
-	return '@';
-    else if (c == DEL_CODE)
+    if (c == DEL_CODE)
 	return '?';
     else if (c == -97)
 	return '=';
@@ -250,9 +232,11 @@ char control_rep(const signed char c)
 }
 
 /* Return the visible representation of multibyte control character c. */
-char control_mbrep(const char *c)
+char control_mbrep(const char *c, bool isdata)
 {
-    assert(c != NULL);
+    /* An embedded newline is an encoded NUL if it is data. */
+    if (*c == '\n' && (isdata || as_an_at))
+	return '@';
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
@@ -325,27 +309,25 @@ int mbwidth(const char *c)
 	return 1;
 }
 
-/* Return the maximum width in bytes of a multibyte character. */
+/* Return the maximum length (in bytes) of a character. */
 int mb_cur_max(void)
 {
-    return
 #ifdef ENABLE_UTF8
-	use_utf8 ? MB_CUR_MAX :
+    if (use_utf8)
+	return MB_CUR_MAX;
+    else
 #endif
-	1;
+	return 1;
 }
 
-/* Convert the Unicode value in chr to a multibyte character with the
- * same wide character value as chr, if possible.  If the conversion
- * succeeds, return the (dynamically allocated) multibyte character and
- * its length.  Otherwise, return an undefined (dynamically allocated)
- * multibyte character and a length of zero. */
+/* Convert the Unicode value in chr to a multibyte character, if possible.
+ * If the conversion succeeds, return the (dynamically allocated) multibyte
+ * character and its length.  Otherwise, return an undefined (dynamically
+ * allocated) multibyte character and a length of zero. */
 char *make_mbchar(long chr, int *chr_mb_len)
 {
     char *chr_mb;
 
-    assert(chr_mb_len != NULL);
-
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	chr_mb = charalloc(MB_CUR_MAX);
@@ -353,7 +335,7 @@ char *make_mbchar(long chr, int *chr_mb_len)
 
 	/* Reject invalid Unicode characters. */
 	if (*chr_mb_len < 0 || !is_valid_unicode((wchar_t)chr)) {
-	    wctomb_reset();
+	    IGNORE_CALL_RESULT(wctomb(NULL, 0));
 	    *chr_mb_len = 0;
 	}
     } else
@@ -368,37 +350,35 @@ char *make_mbchar(long chr, int *chr_mb_len)
 
 /* Parse a multibyte character from buf.  Return the number of bytes
  * used.  If chr isn't NULL, store the multibyte character in it.  If
- * col isn't NULL, store the new display width in it.  If *buf is '\t',
- * we expect col to have the current display width. */
+ * col isn't NULL, add the character's width (in columns) to it. */
 int parse_mbchar(const char *buf, char *chr, size_t *col)
 {
-    int buf_mb_len;
+    int length;
 
     assert(buf != NULL);
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	/* Get the number of bytes in the multibyte character. */
-	buf_mb_len = mblen(buf, MB_CUR_MAX);
+	length = mblen(buf, MB_CUR_MAX);
 
 	/* When the multibyte sequence is invalid, only take the first byte. */
-	if (buf_mb_len < 0) {
+	if (length <= 0) {
 	    IGNORE_CALL_RESULT(mblen(NULL, 0));
-	    buf_mb_len = 1;
-	} else if (buf_mb_len == 0)
-	    buf_mb_len++;
+	    length = 1;
+	}
 
 	/* When requested, store the multibyte character in chr. */
 	if (chr != NULL) {
 	    int i;
 
-	    for (i = 0; i < buf_mb_len; i++)
+	    for (i = 0; i < length; i++)
 		chr[i] = buf[i];
 	}
 
-	/* When requested, store the width of the wide character in col. */
+	/* When requested, add the width of the character to col. */
 	if (col != NULL) {
-	    /* If we have a tab, get its width in columns using the
+	    /* If we have a tab, compute its width in columns based on the
 	     * current value of col. */
 	    if (*buf == '\t')
 		*col += tabsize - *col % tabsize;
@@ -406,8 +386,7 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
 	     * column for the "^", and one for the visible character. */
 	    else if (is_cntrl_mbchar(buf)) {
 		*col += 2;
-	    /* If we have a normal character, get its width in columns
-	     * normally. */
+	    /* If we have a normal character, get its width normally. */
 	    } else
 		*col += mbwidth(buf);
 	}
@@ -415,15 +394,15 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
 #endif
     {
 	/* A byte character is one byte long. */
-	buf_mb_len = 1;
+	length = 1;
 
 	/* When requested, store the byte character in chr. */
 	if (chr != NULL)
 	    *chr = *buf;
 
-	/* When requested, store the width of the wide character in col. */
+	/* When requested, add the width of the character to col. */
 	if (col != NULL) {
-	    /* If we have a tab, get its width in columns using the
+	    /* If we have a tab, compute its width in columns using the
 	     * current value of col. */
 	    if (*buf == '\t')
 		*col += tabsize - *col % tabsize;
@@ -437,7 +416,7 @@ int parse_mbchar(const char *buf, char *chr, size_t *col)
 	}
     }
 
-    return buf_mb_len;
+    return length;
 }
 
 /* Return the index in buf of the beginning of the multibyte character
@@ -492,8 +471,6 @@ int nstrncasecmp(const char *s1, const char *s2, size_t n)
     if (s1 == s2)
 	return 0;
 
-    assert(s1 != NULL && s2 != NULL);
-
     for (; *s1 != '\0' && *s2 != '\0' && n > 0; s1++, s2++, n--) {
 	if (tolower(*s1) != tolower(*s2))
 	    break;
@@ -510,8 +487,6 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
     if (use_utf8) {
 	wchar_t wc1, wc2;
 
-	assert(s1 != NULL && s2 != NULL);
-
 	while (*s1 != '\0' && *s2 != '\0' && n > 0) {
 	    bool bad1 = FALSE, bad2 = FALSE;
 
@@ -555,8 +530,6 @@ char *nstrcasestr(const char *haystack, const char *needle)
 {
     size_t needle_len;
 
-    assert(haystack != NULL && needle != NULL);
-
     if (*needle == '\0')
 	return (char *)haystack;
 
@@ -580,8 +553,6 @@ char *mbstrcasestr(const char *haystack, const char *needle)
     if (use_utf8) {
 	size_t needle_len;
 
-	assert(haystack != NULL && needle != NULL);
-
 	if (*needle == '\0')
 	    return (char *)haystack;
 
@@ -602,27 +573,25 @@ char *mbstrcasestr(const char *haystack, const char *needle)
 
 /* This function is equivalent to strstr(), except in that it scans the
  * string in reverse, starting at rev_start. */
-char *revstrstr(const char *haystack, const char *needle, const char
-	*rev_start)
+char *revstrstr(const char *haystack, const char *needle,
+	const char *pointer)
 {
-    size_t rev_start_len, needle_len;
+    size_t needle_len = strlen(needle);
+    size_t tail_len = strlen(pointer);
 
-    assert(haystack != NULL && needle != NULL && rev_start != NULL);
-
-    if (*needle == '\0')
-	return (char *)rev_start;
-
-    needle_len = strlen(needle);
+    if (needle_len == 0)
+	return (char *)pointer;
 
     if (strlen(haystack) < needle_len)
 	return NULL;
 
-    rev_start_len = strlen(rev_start);
+    if (tail_len < needle_len)
+	pointer += tail_len - needle_len;
 
-    for (; rev_start >= haystack; rev_start--, rev_start_len++) {
-	if (rev_start_len >= needle_len && strncmp(rev_start, needle,
-		needle_len) == 0)
-	    return (char *)rev_start;
+    while (pointer >= haystack) {
+	if (strncmp(pointer, needle, needle_len) == 0)
+	    return (char *)pointer;
+	pointer--;
     }
 
     return NULL;
@@ -635,8 +604,6 @@ char *revstrcasestr(const char *haystack, const char *needle, const char
 {
     size_t rev_start_len, needle_len;
 
-    assert(haystack != NULL && needle != NULL && rev_start != NULL);
-
     if (*needle == '\0')
 	return (char *)rev_start;
 
@@ -665,8 +632,6 @@ char *mbrevstrcasestr(const char *haystack, const char *needle, const
     if (use_utf8) {
 	size_t rev_start_len, needle_len;
 
-	assert(haystack != NULL && needle != NULL && rev_start != NULL);
-
 	if (*needle == '\0')
 	    return (char *)rev_start;
 
@@ -706,8 +671,6 @@ size_t nstrnlen(const char *s, size_t maxlen)
 {
     size_t n = 0;
 
-    assert(s != NULL);
-
     for (; *s != '\0' && maxlen > 0; s++, maxlen--, n++)
 	;
 
@@ -718,8 +681,6 @@ size_t nstrnlen(const char *s, size_t maxlen)
 /* This function is equivalent to strnlen() for multibyte strings. */
 size_t mbstrnlen(const char *s, size_t maxlen)
 {
-    assert(s != NULL);
-
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	size_t n = 0;
@@ -743,7 +704,7 @@ char *mbstrchr(const char *s, const char *c)
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	bool bad_s_mb = FALSE, bad_c_mb = FALSE;
-	char *s_mb = charalloc(MB_CUR_MAX);
+	char symbol[MB_CUR_MAX];
 	const char *q = s;
 	wchar_t ws, wc;
 
@@ -754,9 +715,9 @@ char *mbstrchr(const char *s, const char *c)
 	}
 
 	while (*s != '\0') {
-	    int s_mb_len = parse_mbchar(s, s_mb, NULL);
+	    int sym_len = parse_mbchar(s, symbol, NULL);
 
-	    if (mbtowc(&ws, s_mb, s_mb_len) < 0) {
+	    if (mbtowc(&ws, symbol, sym_len) < 0) {
 		mbtowc_reset();
 		ws = (unsigned char)*s;
 		bad_s_mb = TRUE;
@@ -765,12 +726,10 @@ char *mbstrchr(const char *s, const char *c)
 	    if (bad_s_mb == bad_c_mb && ws == wc)
 		break;
 
-	    s += s_mb_len;
-	    q += s_mb_len;
+	    s += sym_len;
+	    q += sym_len;
 	}
 
-	free(s_mb);
-
 	if (*s == '\0')
 	    q = NULL;
 
@@ -785,8 +744,6 @@ char *mbstrchr(const char *s, const char *c)
 /* This function is equivalent to strpbrk() for multibyte strings. */
 char *mbstrpbrk(const char *s, const char *accept)
 {
-    assert(s != NULL && accept != NULL);
-
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	for (; *s != '\0'; s += move_mbright(s, 0)) {
@@ -857,8 +814,6 @@ char *mbrevstrpbrk(const char *s, const char *accept, const char
  * and FALSE otherwise. */
 bool has_blank_chars(const char *s)
 {
-    assert(s != NULL);
-
     for (; *s != '\0'; s++) {
 	if (isblank(*s))
 	    return TRUE;
@@ -871,25 +826,18 @@ bool has_blank_chars(const char *s)
  * multibyte characters, and FALSE otherwise. */
 bool has_blank_mbchars(const char *s)
 {
-    assert(s != NULL);
-
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	bool retval = FALSE;
-	char *chr_mb = charalloc(MB_CUR_MAX);
+	char symbol[MB_CUR_MAX];
 
 	for (; *s != '\0'; s += move_mbright(s, 0)) {
-	    parse_mbchar(s, chr_mb, NULL);
+	    parse_mbchar(s, symbol, NULL);
 
-	    if (is_blank_mbchar(chr_mb)) {
-		retval = TRUE;
-		break;
-	    }
+	    if (is_blank_mbchar(symbol))
+		return TRUE;
 	}
 
-	free(chr_mb);
-
-	return retval;
+	return FALSE;
     } else
 #endif
 	return has_blank_chars(s);
@@ -912,12 +860,11 @@ bool is_valid_unicode(wchar_t wc)
  * is, and FALSE otherwise. */
 bool is_valid_mbstring(const char *s)
 {
-    assert(s != NULL);
-
-    return
 #ifdef ENABLE_UTF8
-	use_utf8 ? (mbstowcs(NULL, s, 0) != (size_t)-1) :
+    if (use_utf8)
+	return (mbstowcs(NULL, s, 0) != (size_t)-1);
+    else
 #endif
-	TRUE;
+	return TRUE;
 }
 #endif /* !DISABLE_NANORC */
