@@ -60,13 +60,13 @@ void cut_line(void)
 #ifndef NANO_TINY
 /* Move all currently marked text into the cutbuffer, and set the
  * current place we want to where the text used to start. */
-void cut_marked(void)
+void cut_marked(bool *right_side_up)
 {
     filestruct *top, *bot;
     size_t top_x, bot_x;
 
     mark_order((const filestruct **)&top, &top_x,
-		(const filestruct **)&bot, &bot_x, NULL);
+		(const filestruct **)&bot, &bot_x, right_side_up);
 
     move_to_filestruct(&cutbuffer, &cutbottom, top, top_x, bot, bot_x);
     openfile->placewewant = xplustabs();
@@ -82,8 +82,6 @@ void cut_to_eol(void)
 {
     size_t data_len = strlen(openfile->current->data);
 
-    assert(openfile->current_x <= data_len);
-
     if (openfile->current_x < data_len)
 	/* If we're not at the end of the line, move all the text from
 	 * the current position up to it, not counting the newline at
@@ -124,45 +122,39 @@ void do_cut_text(bool copy_text, bool cut_till_eof)
 	/* The length of the string at the current end of the cutbuffer,
 	 * before we add text to it. */
     bool old_no_newlines = ISSET(NO_NEWLINES);
+    bool right_side_up = TRUE;
+	/* There *is* no region, *or* it is marked forward. */
 #endif
+    size_t was_totsize = openfile->totsize;
 
-    assert(openfile->current != NULL && openfile->current->data != NULL);
-
-    /* Empty the cutbuffer when a chain of cuts is broken. */
+    /* If a chain of cuts was broken, empty the cutbuffer. */
     if (!keep_cutbuffer) {
 	free_filestruct(cutbuffer);
 	cutbuffer = NULL;
 #ifdef DEBUG
 	fprintf(stderr, "Blew away cutbuffer =)\n");
 #endif
+	/* Indicate that future cuts should add to the cutbuffer. */
+	keep_cutbuffer = TRUE;
     }
 
 #ifndef NANO_TINY
     if (copy_text) {
+	/* If the cutbuffer isn't empty, remember where it currently ends. */
 	if (cutbuffer != NULL) {
-	    /* If the cutbuffer isn't empty, save where it currently
-	     * ends.  This is where we'll add the new text. */
 	    cb_save = cutbottom;
 	    cb_save_len = strlen(cutbottom->data);
 	}
-
-	/* Set NO_NEWLINES to TRUE, so that we don't disturb the last
-	 * line of the file when moving text to the cutbuffer. */
+	/* Don't add a magicline when moving text to the cutbuffer. */
 	SET(NO_NEWLINES);
     }
-#endif
 
-    /* Ensure that the text we're going to move into the cutbuffer will
-     * be added to the text already there, instead of replacing it. */
-    keep_cutbuffer = TRUE;
-
-#ifndef NANO_TINY
     if (cut_till_eof) {
 	/* Move all text up to the end of the file into the cutbuffer. */
 	cut_to_eof();
     } else if (openfile->mark_set) {
 	/* Move the marked text to the cutbuffer, and turn the mark off. */
-	cut_marked();
+	cut_marked(&right_side_up);
 	openfile->mark_set = FALSE;
     } else if (ISSET(CUT_TO_END))
 	/* Move all text up to the end of the line into the cutbuffer. */
@@ -174,10 +166,9 @@ void do_cut_text(bool copy_text, bool cut_till_eof)
 
 #ifndef NANO_TINY
     if (copy_text) {
-	/* Copy the text in the cutbuffer, starting at its saved end if
-	 * there is one, back into the filestruct.  This effectively
-	 * uncuts the text we just cut without marking the file as
-	 * modified. */
+	/* Copy the text that is in the cutbuffer (starting at its saved end,
+	 * if there is one) back into the current buffer.  This effectively
+	 * uncuts the text we just cut. */
 	if (cutbuffer != NULL) {
 	    if (cb_save != NULL) {
 		cb_save->data += cb_save_len;
@@ -186,17 +177,18 @@ void do_cut_text(bool copy_text, bool cut_till_eof)
 	    } else
 		copy_from_filestruct(cutbuffer);
 
-	    /* Set the current place we want to where the text from the
-	     * cutbuffer ends. */
-	    openfile->placewewant = xplustabs();
+	    /* If the copied region was marked forward, put the new desired
+	     * x position at its end; otherwise, leave it at its beginning. */
+	    if (right_side_up)
+		openfile->placewewant = xplustabs();
 	}
-
-	/* Set NO_NEWLINES back to what it was before, since we're done
-	 * disturbing the text. */
+	/* Restore the magicline behavior now that we're done fiddling. */
 	if (!old_no_newlines)
 	    UNSET(NO_NEWLINES);
     } else
 #endif /* !NANO_TINY */
+    /* Only set the modification flag if actually something was cut. */
+    if (openfile->totsize != was_totsize)
 	set_modified();
 
     refresh_needed = TRUE;
@@ -264,11 +256,9 @@ void do_cut_till_eof(void)
 /* Copy text from the cutbuffer into the current filestruct. */
 void do_uncut_text(void)
 {
-    int was_lineno = openfile->current->lineno;
+    ssize_t was_lineno = openfile->current->lineno;
 
-    assert(openfile->current != NULL && openfile->current->data != NULL);
-
-    /* If the cutbuffer is empty, get out. */
+    /* If the cutbuffer is empty, there is nothing to do. */
     if (cutbuffer == NULL)
 	return;
 
@@ -287,13 +277,17 @@ void do_uncut_text(void)
     if (openfile->current->lineno - was_lineno < editwinrows)
 	focusing = FALSE;
 
-    /* Set the current place we want to where the text from the
-     * cutbuffer ends. */
+    /* Set the desired x position to where the pasted text ends. */
     openfile->placewewant = xplustabs();
 
     /* Mark the file as modified. */
     set_modified();
 
+    /* Update the cursor position to account for the inserted lines. */
+    reset_cursor();
+
+    ensure_line_is_visible();
+
     refresh_needed = TRUE;
 
 #ifndef DISABLE_COLOR
