@@ -104,7 +104,7 @@ int do_statusbar_input(bool *ran_func, bool *finished)
     if ((have_shortcut || get_key_buffer_len() == 0) && kbinput != NULL) {
 	/* Inject all characters in the input buffer at once, filtering out
 	 * control characters. */
-	do_statusbar_output(kbinput, kbinput_len, TRUE, NULL);
+	do_statusbar_output(kbinput, kbinput_len, TRUE);
 
 	/* Empty the input buffer. */
 	kbinput_len = 0;
@@ -139,18 +139,7 @@ int do_statusbar_input(bool *ran_func, bool *finished)
 				s->scfunc == do_backspace))
 	    ;
 	else if (s->scfunc == do_verbatim_input) {
-	    bool got_newline = FALSE;
-		/* Whether we got a verbatim ^J. */
-
-	    do_statusbar_verbatim_input(&got_newline);
-
-	    /* If we got a verbatim ^J, remove it from the input buffer,
-	     * fake a press of Enter, and indicate that we're done. */
-	    if (got_newline) {
-		get_input(NULL, 1);
-		input = sc_seq_or(do_enter, 0);
-		*finished = TRUE;
-	    }
+	    do_statusbar_verbatim_input();
 	} else if (s->scfunc == do_cut_text_void)
 	    do_statusbar_cut_text();
 	else if (s->scfunc == do_delete)
@@ -202,11 +191,10 @@ int do_statusbar_mouse(void)
 }
 #endif
 
-/* The user typed input_len multibyte characters.  Add them to the
- * statusbar prompt, setting got_newline to TRUE if we got a verbatim ^J,
- * and filtering out ASCII control characters if filtering is TRUE. */
+/* The user typed input_len multibyte characters.  Add them to the answer,
+ * filtering out ASCII control characters if filtering is TRUE. */
 void do_statusbar_output(int *the_input, size_t input_len,
-	bool filtering, bool *got_newline)
+	bool filtering)
 {
     char *output = charalloc(input_len + 1);
     char *char_buf = charalloc(mb_cur_max());
@@ -220,18 +208,9 @@ void do_statusbar_output(int *the_input, size_t input_len,
     i = 0;
 
     while (i < input_len) {
-	/* When not filtering, convert nulls and stop at a newline. */
-	if (!filtering) {
-	    if (output[i] == '\0')
-		output[i] = '\n';
-	    else if (output[i] == '\n') {
-		/* Put back the rest of the characters for reparsing,
-		 * indicate that we got a ^J and get out. */
-		unparse_kbinput(output + i, input_len - i);
-		*got_newline = TRUE;
-		return;
-	    }
-	}
+	/* Encode any NUL byte as 0x0A. */
+	if (output[i] == '\0')
+	    output[i] = '\n';
 
 	/* Interpret the next multibyte character. */
 	char_len = parse_mbchar(output + i, char_buf, NULL);
@@ -306,7 +285,6 @@ void do_statusbar_delete(void)
 
 	charmove(answer + statusbar_x, answer + statusbar_x + char_len,
 			strlen(answer) - statusbar_x - char_len + 1);
-	align(&answer);
 
 	update_the_statusbar();
     }
@@ -370,19 +348,15 @@ void do_statusbar_prev_word(void)
 }
 #endif /* !NANO_TINY */
 
-/* Get verbatim input, setting got_newline to TRUE if we get a ^J as
- * part of the verbatim input. */
-void do_statusbar_verbatim_input(bool *got_newline)
+/* Get verbatim input and inject it into the answer, without filtering. */
+void do_statusbar_verbatim_input(void)
 {
     int *kbinput;
     size_t kbinput_len;
 
-    /* Read in all the verbatim characters. */
     kbinput = get_verbatim_kbinput(bottomwin, &kbinput_len);
 
-    /* Display all the verbatim characters at once, not filtering out
-     * control characters. */
-    do_statusbar_output(kbinput, kbinput_len, FALSE, got_newline);
+    do_statusbar_output(kbinput, kbinput_len, FALSE);
 }
 
 /* Return the zero-based column position of the cursor in the answer. */
@@ -459,9 +433,7 @@ void update_the_statusbar(void)
 
 /* Get a string of input at the statusbar prompt. */
 functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
-#ifndef DISABLE_TABCOMP
 	bool allow_files, bool *listed,
-#endif
 #ifndef DISABLE_HISTORIES
 	filestruct **history_list,
 #endif
@@ -532,7 +504,7 @@ functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
 	if (func == do_tab) {
 #ifndef DISABLE_HISTORIES
 	    if (history_list != NULL) {
-		if (last_kbinput != sc_seq_or(do_tab, TAB_CODE))
+		if (last_kbinput != the_code_for(do_tab, TAB_CODE))
 		    complete_len = strlen(answer);
 
 		if (complete_len > 0) {
@@ -639,10 +611,7 @@ functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
  * The allow_tabs parameter indicates whether we should allow tabs to be
  * interpreted.  The allow_files parameter indicates whether we should
  * allow all files (as opposed to just directories) to be tab completed. */
-int do_prompt(bool allow_tabs,
-#ifndef DISABLE_TABCOMP
-	bool allow_files,
-#endif
+int do_prompt(bool allow_tabs, bool allow_files,
 	int menu, const char *curranswer,
 #ifndef DISABLE_HISTORIES
 	filestruct **history_list,
@@ -652,9 +621,7 @@ int do_prompt(bool allow_tabs,
     va_list ap;
     int retval;
     functionptrtype func = NULL;
-#ifndef DISABLE_TABCOMP
     bool listed = FALSE;
-#endif
     /* Save a possible current statusbar x position. */
     size_t was_statusbar_x = statusbar_x;
 
@@ -672,10 +639,7 @@ int do_prompt(bool allow_tabs,
     /* Reserve five columns for colon plus angles plus answer, ":<aa>". */
     null_at(&prompt, actual_x(prompt, (COLS < 5) ? 0 : COLS - 5));
 
-    func = acquire_an_answer(&retval, allow_tabs,
-#ifndef DISABLE_TABCOMP
-			allow_files, &listed,
-#endif
+    func = acquire_an_answer(&retval, allow_tabs, allow_files, &listed,
 #ifndef DISABLE_HISTORIES
 			history_list,
 #endif
@@ -724,6 +688,8 @@ int do_prompt(bool allow_tabs,
 int do_yesno_prompt(bool all, const char *msg)
 {
     int response = -2, width = 16;
+    char *message = display_string(msg, 0, COLS, FALSE);
+
     /* TRANSLATORS: For the next three strings, if possible, specify
      * the single-byte shortcuts for both your language and English.
      * For example, in French: "OoYy", for both "Oui" and "Yes". */
@@ -735,12 +701,9 @@ int do_yesno_prompt(bool all, const char *msg)
      * that are accepted for the corresponding answer.  Of each variable,
      * the first character is displayed in the help lines. */
 
-    do {
+    while (response == -2) {
 	int kbinput;
 	functionptrtype func;
-#ifndef DISABLE_MOUSE
-	int mouse_x, mouse_y;
-#endif
 
 	if (!ISSET(NO_HELP)) {
 	    char shortstr[3];
@@ -774,7 +737,7 @@ int do_yesno_prompt(bool all, const char *msg)
 	/* Color the statusbar over its full width and display the question. */
 	wattron(bottomwin, interface_color_pair[TITLE_BAR]);
 	blank_statusbar();
-	mvwaddnstr(bottomwin, 0, 0, msg, actual_x(msg, COLS - 1));
+	mvwaddnstr(bottomwin, 0, 0, message, actual_x(message, COLS - 1));
 	wattroff(bottomwin, interface_color_pair[TITLE_BAR]);
 
 	wnoutrefresh(bottomwin);
@@ -786,17 +749,13 @@ int do_yesno_prompt(bool all, const char *msg)
 	currmenu = MYESNO;
 	kbinput = get_kbinput(bottomwin);
 
-#ifndef NANO_TINY
-	if (kbinput == KEY_WINCH)
-	    continue;
-#endif
-
 	func = func_from_key(&kbinput);
 
 	if (func == do_cancel)
 	    response = -1;
 #ifndef DISABLE_MOUSE
 	else if (kbinput == KEY_MOUSE) {
+	    int mouse_x, mouse_y;
 	    /* We can click on the Yes/No/All shortcuts to select an answer. */
 	    if (get_mouseinput(&mouse_x, &mouse_y, FALSE) == 0 &&
 			wmouse_trafo(bottomwin, &mouse_y, &mouse_x, FALSE) &&
@@ -826,7 +785,9 @@ int do_yesno_prompt(bool all, const char *msg)
 	    else if (all && strchr(allstr, kbinput) != NULL)
 		response = 2;
 	}
-    } while (response == -2);
+    }
+
+    free(message);
 
     return response;
 }
